@@ -1,6 +1,7 @@
 import boto3
 from .contexts import Contexts
 from .logger import Logger
+from .stages import Stages
 
 session = boto3.session.Session()
 ssm = session.client(
@@ -43,17 +44,39 @@ def load_context_ssm_parameters(project, application, stage, parameter_type, pre
             ### set Scope if it has not previousely been set, i.e. for newly loaded parameters
             if not 'Scope' in details.keys():
                 details['Scope']= Contexts.translate_context(*Contexts.decode_context_path(details['Path'][len(prefix):])[0:3])
+            ### set Stage if it has not previousely been set, i.e. for newly loaded parameters
+            if not 'Stage' in details.keys():
+                details['Stage'] = Stages.shorten('/'.join(Contexts.decode_context_path(details['Path'][len(prefix):])[2:3]))
+            ### set Origin if it has not previousely been set, i.e. for newly loaded parameters
+            if not 'Origin' in details.keys():
+                details['Origin']= {
+                    'Project': Contexts.decode_context_path(details['Path'][len(prefix):])[0],
+                    'Application': Contexts.decode_context_path(details['Path'][len(prefix):])[1],
+                    'Stage': Stages.shorten('/'.join(Contexts.decode_context_path(details['Path'][len(prefix):])[2:3])),
+                }
+
     return parameters
 
 ###--------------------------------------------------------------------------------------------
 
 def locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=None, uninherited=False):
+    result = {}
     paths = list(reversed(Contexts.get_hierachy_context_paths(project, application, stage, key, prefix=prefix, allow_stages=(not stage is None), uninherited=uninherited)))
     for path in paths:
         Logger.debug(f"Describing SSM parameters: path={path}")
         response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
-        if len(response['Parameters']) > 0: return response['Parameters']
-    return []
+        if len(response['Parameters']) > 0:
+            result = response['Parameters'][0]
+            result['Scope'] = Contexts.translate_context(*Contexts.decode_context_path(result['Name'][len(prefix):])[0:3])
+            result['Stage'] = Stages.shorten('/'.join(Contexts.decode_context_path(result['Name'][len(prefix):])[2:3]))
+            result['Origin']= {
+                'Project': Contexts.decode_context_path(result['Name'][len(prefix):])[0],
+                'Application': Contexts.decode_context_path(result['Name'][len(prefix):])[1],
+                'Stage': Stages.shorten('/'.join(Contexts.decode_context_path(result['Name'][len(prefix):])[2:3])),
+            }
+
+            break
+    return result
 
 ###--------------------------------------------------------------------------------------------
 
