@@ -28,7 +28,7 @@ from .dict_utils import *
 
 @click.group(context_settings=DEFAULT_CLICK_CONTEXT)
 def cli():
-    """Mable DSO CLI"""
+    """DSO CLI"""
     pass
 
 ###--------------------------------------------------------------------------------------------
@@ -112,7 +112,7 @@ def version():
     """
     Display version details.
     """
-    click.echo(f"Mable DSO CLI: {__version__}\nPython: {platform.sys.version}")
+    click.echo(f"DSO CLI: {__version__}\nPython: {platform.sys.version}")
 
 
 ###--------------------------------------------------------------------------------------------
@@ -144,6 +144,7 @@ def add_parameter(key, key_option, value, value_option, stage, input, format, ve
 
         if input:
             validate_none_provided([key, key_option], ["KEY", "'-k' / '--key'"], ["'-i' / '--input'"])
+            validate_none_provided([key, key_option], ["VALUE", "'-v' / '--value'"], ["'-i' / '--input'"])
             parameters = read_data(input, 'Parameters', ['Key', 'Value'], format)
 
             ### eat possible enclosing (double) quotes when source is file, stdin has already eaten them!
@@ -156,9 +157,11 @@ def add_parameter(key, key_option, value, value_option, stage, input, format, ve
 
         ### no input file
         else:
+            # key = validate_only_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
             validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
             key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-            validate_not_all_provided([key, key_option], ["VALUE", "'-v' / '--value'"])
+            # value = validate_only_one_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
+            validate_not_all_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
             value = validate_at_least_one_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
             parameters.append({'Key': key, 'Value': value})
 
@@ -214,7 +217,8 @@ def list_parameter(stage, uninherited, filter, query_all, query, format, verbosi
         nonlocal stage, query
         stage = Stages.normalize(stage)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
-        query = validate_query_argument(query, query_all, '{Parameters: Parameters[*].{Key: Key, Value: Value}}')
+        defaultQuery = '{Parameters: Parameters[*].{Key: Key, Value: Value, Scope: Scope, Origin: Origin}}'
+        query = validate_query_argument(query, query_all, defaultQuery)
         if filter:
             try:
                 re.compile(filter)
@@ -264,9 +268,13 @@ def get_parameter(key, key_option, stage, revision, history, query_all, query, f
         validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         validate_not_all_provided([history, revision], ["'--history'", "'--revision'"])
-        query = validate_query_argument(query, query_all, '{Revisions: Revisions[*].{RevisionId: RevisionId, Value: Value, Date: Date}}' if history else '{Value: Value}')
+        if history:
+            defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Value: Value, Scope: Scope, Origin: Origin}}'
+        else:
+            defaultQuery = '{Value: Value}'
+        query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
-        
+
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
@@ -451,7 +459,8 @@ def list_secret(stage, uninherited, decrypt, filter, query_all, query, format, v
     def validate_command_usage():
         nonlocal stage, query
         stage = Stages.normalize(stage)
-        query = validate_query_argument(query, query_all, '{Secrets: Secrets[*].{Key: Key, Value: Value}}')
+        defaultQuery = '{Secrets: Secrets[*].{Key: Key, Value: Value, Scope: Scope, Origin: Origin}}'
+        query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
         if filter:
             try:
@@ -502,7 +511,11 @@ def get_secret(key, key_option, stage, revision, history, query_all, query, form
         validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         validate_not_all_provided([history, revision], ["'--history'", "'--revision'"])
-        query = validate_query_argument(query, query_all, '{Revisions: Revisions[*].{RevisionId: RevisionId, Value: Value, Date: Date}}' if history else '{Value: Value}')
+        if history:
+            defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Value: Value, Scope: Scope, Origin: Origin}}'
+        else:
+            defaultQuery = '{Value: Value}'
+        query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
 
@@ -600,6 +613,7 @@ def delete_secret(key, key_option, input, format, stage, verbosity, global_scope
 @command_doc(CLI_COMMANDS_HELP['template']['list'])
 @click.option('-u','--uninherited', 'uninherited', is_flag=True, default=False, help=f"{CLI_PARAMETERS_HELP['template']['uninherited']}")
 @click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
+@click.option('--include-contents', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['template']['include_contents']}")
 @click.option('--filter', required=False, metavar='<regex>', help=f"{CLI_PARAMETERS_HELP['common']['filter']}")
 @click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
 @click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
@@ -609,12 +623,16 @@ def delete_secret(key, key_option, input, format, stage, verbosity, global_scope
 @click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
 @click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
 @click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
-def list_template(uninherited, stage, filter, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+def list_template(uninherited, stage, include_contents, filter, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
         nonlocal stage, query
         stage = Stages.normalize(stage)
-        query = validate_query_argument(query, query_all, '{Templates: Templates[*].{Key: Key, RenderTo: RenderTo}}')
+        if include_contents:
+            defaultQuery = '{Templates: Templates[*].{Key: Key, Scope: Scope, Origin: Origin, RenderTo: RenderTo, Contents: Contents}}'
+        else:
+            defaultQuery = '{Templates: Templates[*].{Key: Key, Scope: Scope, Origin: Origin, RenderTo: RenderTo}}'
+        query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
         if filter:
             try:
@@ -628,56 +646,7 @@ def list_template(uninherited, stage, filter, query_all, query, format, verbosit
         Configs.load(working_dir if working_dir else os.getcwd(),
                         'global' if global_scope else 'project' if project_scope else 'application',
                         config_override)
-        result = Templates.list(stage, uninherited, filter)
-        output = format_data(result, query, format)
-        if output: Pager.page(output)
-    except DSOException as e:
-        Logger.error(e.message)
-    except Exception as e:
-        msg = getattr(e, 'message', getattr(e, 'msg', str(e)))
-        Logger.critical(msg)
-        if verbosity >= log_levels['full']:
-            raise
-
-###--------------------------------------------------------------------------------------------
-
-@template.command('get', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['template']['get']}")
-@command_doc(CLI_COMMANDS_HELP['template']['get'])
-@click.argument('key', required=False)
-@click.option('-k', '--key', 'key_option', metavar='<key>', required=False, help=f"{CLI_PARAMETERS_HELP['template']['key']}")
-@click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
-@click.option('--revision', required=False, help=f"{CLI_PARAMETERS_HELP['parameter']['revision']}")
-@click.option('--history', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['parameter']['history']}")
-@click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
-@click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
-@click.option('-f', '--format', required=False, type=click.Choice(['json', 'yaml', 'raw', 'csv']), default='json', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['format']}")
-@click.option('-b', '--verbosity', metavar='<number>', required=False, type=RangeParamType(click.INT, minimum=0, maximum=5), default='2', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['verbosity']}")
-@click.option('--global-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['global_scope']}")
-@click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
-@click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
-@click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
-def get_template(key, key_option, stage, revision, history, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
-
-    def validate_command_usage():
-        nonlocal stage, key, query
-        stage = Stages.normalize(stage)
-        validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-        key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-        validate_not_all_provided([history, revision], ["'--history'", "'--revision'"])
-        query = validate_query_argument(query, query_all, '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date}}' if history else '{Contents: Contents}')
-        validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
-
-    try:
-        Logger.set_verbosity(verbosity)
-        validate_command_usage()
-        Configs.load(working_dir if working_dir else os.getcwd(),
-                        'global' if global_scope else 'project' if project_scope else 'application',
-                        config_override)
-
-        if history:
-            result = Templates.history(stage, key)
-        else:
-            result = Templates.get(stage, key, revision)
+        result = Templates.list(stage, uninherited, include_contents, filter)
         output = format_data(result, query, format)
         if output: Pager.page(output)
     except DSOException as e:
@@ -823,6 +792,64 @@ def add_template(key, key_option, render_path, input, recursive, stage, verbosit
 
 ###--------------------------------------------------------------------------------------------
 
+@template.command('get', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['template']['get']}")
+@command_doc(CLI_COMMANDS_HELP['template']['get'])
+@click.argument('key', required=False)
+@click.option('-k', '--key', 'key_option', metavar='<key>', required=False, help=f"{CLI_PARAMETERS_HELP['template']['key']}")
+@click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
+@click.option('--revision', required=False, help=f"{CLI_PARAMETERS_HELP['parameter']['revision']}")
+@click.option('--history', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['parameter']['history']}")
+@click.option('--include-contents', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['template']['include_contents']}")
+@click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
+@click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
+@click.option('-f', '--format', required=False, type=click.Choice(['json', 'yaml', 'raw', 'csv']), default='json', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['format']}")
+@click.option('-b', '--verbosity', metavar='<number>', required=False, type=RangeParamType(click.INT, minimum=0, maximum=5), default='2', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['verbosity']}")
+@click.option('--global-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['global_scope']}")
+@click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
+@click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
+@click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
+def get_template(key, key_option, stage, revision, history, include_contents, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+
+    def validate_command_usage():
+        nonlocal stage, key, query
+        stage = Stages.normalize(stage)
+        validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        validate_not_all_provided([history, revision], ["'--history'", "'--revision'"])
+        # validate_not_all_provided([history, revision], ["'--history'", "'--include-contents'"])
+        if history:
+            if include_contents:
+                defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Scope: Scope, Origin: Origin, Contents: Contents}}'
+            else:
+                defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Scope: Scope, Origin: Origin}}'
+        else:
+            defaultQuery = '{Contents: Contents}'
+        query = validate_query_argument(query, query_all, defaultQuery)
+        validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
+
+    try:
+        Logger.set_verbosity(verbosity)
+        validate_command_usage()
+        Configs.load(working_dir if working_dir else os.getcwd(),
+                        'global' if global_scope else 'project' if project_scope else 'application',
+                        config_override)
+
+        if history:
+            result = Templates.history(stage, key, include_contents)
+        else:
+            result = Templates.get(stage, key, revision)
+        output = format_data(result, query, format)
+        if output: Pager.page(output)
+    except DSOException as e:
+        Logger.error(e.message)
+    except Exception as e:
+        msg = getattr(e, 'message', getattr(e, 'msg', str(e)))
+        Logger.critical(msg)
+        if verbosity >= log_levels['full']:
+            raise
+
+###--------------------------------------------------------------------------------------------
+
 @template.command('delete', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['template']['delete']}")
 @command_doc(CLI_COMMANDS_HELP['template']['delete'])
 @click.argument('key', required=False)
