@@ -165,6 +165,7 @@ CLI_PARAMETERS_HELP = {
         'input' : "Path to a local file containing the template content.",
         'recursive' : "Add files recursively.",
         'uninherited': "Select only templates which are specific to the gievn context, i.e. not inherited from the parent contexts.",
+        'include_contents': "Include template contenets in the output. ",
 
     },
     'config': {
@@ -190,11 +191,11 @@ CLI_MESSAGES = {
     'ArgumentsAllProvided': "All of the following arguments/options must be provide: {0}",
     'ArgumentsNoneProvided': "The following arguments/options cannot be provided: {0}",
     'ArgumentsNotAllProvided': "The following arguments/options cannot be provided together: {0}",
-    'ArgumentsOnlyOneProvidedBecause': "Becasue {0} provided, only one of the following arguments/options may be provided: {1}",
-    'ArgumentsAtLeastOneProvidedBecause': "Becasue {0} provided, at least one of the following arguments/options must also be provided: {1}",
-    'ArgumentsAllProvidedBecause': "Becasue {0} provided, all of the following arguments/options must also be provide: {1}",
-    'ArgumentsNoneProvidedBecause': "Becasue {0} provided, the following arguments/options cannot be provided: {1}",
-    'ArgumentsNotAllProvidedBecasue': "Becasue {0} provided, the following arguments/options may be provided together: {1}",
+    'ArgumentsOnlyOneProvidedBecause': "Since {0} provided, only one of the following arguments/options may be provided: {1}",
+    'ArgumentsAtLeastOneProvidedBecause': "Since {0} provided, at least one of the following arguments/options must also be provided: {1}",
+    'ArgumentsAllProvidedBecause': "Since {0} provided, all of the following arguments/options must also be provide: {1}",
+    'ArgumentsNoneProvidedBecause': "Since {0} provided, the following arguments/options cannot be provided: {1}",
+    'ArgumentsNotAllProvidedBecasue': "Since {0} provided, the following arguments/options may be provided together: {1}",
     'TryHelpWithCommand': "Try '{0} --help' for more details.",
     'TryHelp': "Try the command with '-h' / '--help' option for more details.",
     'InvalidFileFormat': "Invalid file, not conforming to the expected '{0}' format.",
