@@ -50,9 +50,10 @@ class AwsSsmParameterProvider(ParameterProvider):
                                     'Key': key, 
                                     'Value': details['Value'], 
                                     'Scope': details['Scope'],
+                                    'Origin': details['Origin'],
                                     'RevisionId': str(details['Version']),
                                     'Date': details['Date'],
-                                    'Version': details['Version'],
+                                    # 'Version': details['Version'],
                                     'Path': details['Path'],
                                     })
 
@@ -65,7 +66,7 @@ class AwsSsmParameterProvider(ParameterProvider):
         assert_ssm_parameter_no_namespace_overwrites(project, application, stage, key, prefix=self.get_prefix())
         Logger.debug(f"Locating SSM parameter: project={project}, application={application}, stage={stage}, key={key}")
         found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix(), uninherited=True)
-        if found and not found[0]['Type'] == 'String':
+        if found and not found['Type'] == 'String':
             raise DSOException(f"Failed to add parameter '{key}' becasue becasue the key is not available in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         path = get_ssm_path(project, application, stage=stage, key=key, prefix=self.get_prefix())
         Logger.info(f"Adding SSM parameter: path={path}")
@@ -74,8 +75,13 @@ class AwsSsmParameterProvider(ParameterProvider):
                 'Key': key, 
                 'Value': value,
                 'Scope': Contexts.translate_context(project, application, stage),
+                'Origin': {
+                    'Project': project,
+                    'Application': application,
+                    'Stage': Stages.shorten(stage),
+                },
                 'RevisionId': str(response['Version']),
-                'Version': response['Version'],
+                # 'Version': response['Version'],
                 'Path': path,
                 }
 
@@ -87,22 +93,23 @@ class AwsSsmParameterProvider(ParameterProvider):
         if not found:
             raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] == 'String':
+            if not found['Type'] == 'String':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        Logger.info(f"Getting SSM parameter: path={found[0]['Name']}")
-        response = ssm.get_parameter_history(Name=found[0]['Name'])
+        Logger.info(f"Getting SSM parameter: path={found['Name']}")
+        response = ssm.get_parameter_history(Name=found['Name'])
         parameters = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         if revision is None:
             ### get the latest revision
             result = {
                     'RevisionId': str(parameters[0]['Version']),
+                    'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'Key': key, 
                     'Value': parameters[0]['Value'],
-                    'Scope': Contexts.translate_context(project, application, stage),
-                    'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'Scope': found['Scope'],
+                    'Origin': found['Origin'],
                     'User': parameters[0]['LastModifiedUser'],
-                    'Version': parameters[0]['Version'],
-                    'Path': found[0]['Name'],
+                    # 'Version': parameters[0]['Version'],
+                    'Path': found['Name'],
                     }
         else:
             ### get specific revision
@@ -111,13 +118,14 @@ class AwsSsmParameterProvider(ParameterProvider):
                 raise DSOException(f"Revision '{revision}' not found for parameter '{key}' in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
             result = {
                     'RevisionId':str(parameters[0]['Version']),
+                    'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'Key': key, 
                     'Value': parameters[0]['Value'],
-                    'Scope': Contexts.translate_context(project, application, stage),
-                    'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'Scope': found['Scope'],
+                    'Origin': found['Origin'],
                     'User': parameters[0]['LastModifiedUser'],
-                    'Version': parameters[0]['Version'],
-                    'Path': found[0]['Name'],
+                    # 'Version': parameters[0]['Version'],
+                    'Path': found['Name'],
                     }
 
         return result
@@ -130,21 +138,22 @@ class AwsSsmParameterProvider(ParameterProvider):
         if not found:
             raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] == 'String':
+            if not found['Type'] == 'String':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        Logger.info(f"Getting SSM parameter: path={found[0]['Name']}")
-        response = ssm.get_parameter_history(Name=found[0]['Name'])
+        Logger.info(f"Getting SSM parameter: path={found['Name']}")
+        response = ssm.get_parameter_history(Name=found['Name'])
         parameters = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         result = { "Revisions":
             [{
                 'RevisionId': str(parameter['Version']),
+                'Date': parameter['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                 'Key': key,
                 'Value': parameter['Value'],
-                'Scope': Contexts.translate_context(project, application, stage),
-                'Date': parameter['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                'Scope': found['Scope'],
+                'Origin': found['Origin'],
                 'User': parameter['LastModifiedUser'],
-                'Version': parameter['Version'],
-                'Path': found[0]['Name'],
+                # 'Version': parameter['Version'],
+                'Path': found['Name'],
             } for parameter in parameters]
         }
 
@@ -160,15 +169,16 @@ class AwsSsmParameterProvider(ParameterProvider):
             raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
             # if len(found) > 1:
-            #     Logger.warn(f"More than one parameter found at '{found[0]['Name']}'. The first one taken, and the rest were discarded.")
-            if not found[0]['Type'] == 'String':
+            #     Logger.warn(f"More than one parameter found at '{found['Name']}'. The first one taken, and the rest were discarded.")
+            if not found['Type'] == 'String':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        Logger.info(f"Deleting SSM parameter: path={found[0]['Name']}")
-        ssm.delete_parameter(Name=found[0]['Name'])
+        Logger.info(f"Deleting SSM parameter: path={found['Name']}")
+        ssm.delete_parameter(Name=found['Name'])
         return {
                 'Key': key, 
-                'Scope': Contexts.translate_context(project, application, stage),
-                'Path': found[0]['Name'],
+                'Scope': found['Scope'],
+                'Origin': found['Origin'],
+                'Path': found['Name'],
                 }
 
 ###--------------------------------------------------------------------------------------------
