@@ -40,34 +40,6 @@ class AwsSsmSecretProvider(SecretProvider):
     def get_prefix(self):
         return Configs.secret_spec('prefix')
 
-###--------------------------------------------------------------------------------------------
-
-    def assert_no_scope_overwrites(self, project, application, stage, key):
-        """
-            check if a secret will overwrite parent or childern secrets (with the same scopes) in the same stage (always uninherited)
-            e.g.: 
-                secret a.b.c would overwrite a.b (super scope)
-                secret a.b would overwrite a.b.c (sub scope)
-        """
-        Logger.debug(f"Checking secret overwrites: project={project}, application={application}, stage={stage}, key={key}")
-        
-        ### check children secrets
-        path = self.get_prefix(project, application, stage, key)
-        response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
-        if len(response['Parameters']) > 0:
-            raise DSOException("Secret key '{0}' is not allowed in the given context becasue it would overwrite '{0}.{1}' and all other secrets in '{0}.*' scope if any.".format(key,response['Parameters'][0]['Name'][len(path)+1:]))
-
-        ### check parent secrets
-        scopes = key.split('.')
-        for n in range(len(scopes)-1):
-            subKey = '.'.join(scopes[0:n+1])
-            path = self.get_prefix(project, application, stage, subKey)
-            Logger.debug(f"Describing secrets: path={path}")
-            # secrets = ssm.describe_parameters(ParameterFilters=[{'Key':'Type', 'Values':['SecureString']},{'Key':'Name', 'Values':[path]}])
-            response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
-            if len(response['Parameters']) > 0:
-                raise DSOException("Secret key '{0}' is not allowed in the given context becasue it would overwrite secret '{1}'.".format(key, subKey))
-
 ###--------------------------------------------------------------------------------------------
 
     def list(self, project, application, stage, uninherited=False, decrypt=False, filter=None):
@@ -79,9 +51,9 @@ class AwsSsmSecretProvider(SecretProvider):
                                 'Key': key, 
                                 'Value': details['Value'], 
                                 'Scope': details['Scope'],
+                                'Origin': details['Origin'],
                                 'RevisionId': str(details['Version']),
                                 'Date': details['Date'],
-                                'Version': details['Version'],
                                 'Path': details['Path'],
                                 })
 
@@ -94,7 +66,7 @@ class AwsSsmSecretProvider(SecretProvider):
         assert_ssm_parameter_no_namespace_overwrites(project, application, stage, key, prefix=self.get_prefix())
         Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
         found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix(), uninherited=True)
-        if found and not found[0]['Type'] == 'SecureString':
+        if found and not found['Type'] == 'SecureString':
             raise DSOException(f"Failed to add secret '{key}' becasue becasue the key is not available in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         path = get_ssm_path(project, application, stage=stage, key=key, prefix=self.get_prefix())
         Logger.info(f"Adding SSM secret: path={path}")
@@ -103,8 +75,12 @@ class AwsSsmSecretProvider(SecretProvider):
                 'Key': key, 
                 'Value': value,
                 'Scope': Contexts.translate_context(project, application, stage),
+                'Origin': {
+                    'Project': project,
+                    'Application': application,
+                    'Stage': Stages.shorten(stage),
+                },
                 'RevisionId': str(response['Version']),
-                'Version': response['Version'],
                 'Path': path,
                 }
 
@@ -116,22 +92,22 @@ class AwsSsmSecretProvider(SecretProvider):
         if not found:
             raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] == 'SecureString':
+            if not found['Type'] == 'SecureString':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        Logger.info(f"Getting SSM secret: path={found[0]['Name']}")
-        response = ssm.get_parameter_history(Name=found[0]['Name'], WithDecryption=True)
+        Logger.info(f"Getting SSM secret: path={found['Name']}")
+        response = ssm.get_parameter_history(Name=found['Name'], WithDecryption=True)
         secrets = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         if revision is None:
             ### get the latest revision
             result = {
                     'RevisionId': str(secrets[0]['Version']),
+                    'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'Key': key, 
                     'Value': secrets[0]['Value'],
-                    'Scope': Contexts.translate_context(project, application, stage),
-                    'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'Scope': found['Scope'],
+                    'Origin': found['Origin'],
                     'User': secrets[0]['LastModifiedUser'],
-                    'Version': secrets[0]['Version'],
-                    'Path': found[0]['Name'],
+                    'Path': found['Name'],
                     }
         else:
             ### get specific revision
@@ -140,13 +116,13 @@ class AwsSsmSecretProvider(SecretProvider):
                 raise DSOException(f"Revision '{revision}' not found for secret '{key}' in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
             result = {
                     'RevisionId': str(secrets[0]['Version']),
+                    'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'Key': key, 
                     'Value': secrets[0]['Value'],
-                    'Scope': Contexts.translate_context(project, application, stage),
-                    'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'Scope': found['Scope'],
+                    'Origin': found['Origin'],
                     'User': secrets[0]['LastModifiedUser'],
-                    'Version': secrets[0]['Version'],
-                    'Path': found[0]['Name'],
+                    'Path': found['Name'],
                     }
 
         return result
@@ -158,21 +134,21 @@ class AwsSsmSecretProvider(SecretProvider):
         if not found:
             raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            if not found[0]['Type'] == 'SecureString':
+            if not found['Type'] == 'SecureString':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        Logger.info(f"Getting SSM secret: path={found[0]['Name']}")
-        response = ssm.get_parameter_history(Name=found[0]['Name'], WithDecryption=True)
+        Logger.info(f"Getting SSM secret: path={found['Name']}")
+        response = ssm.get_parameter_history(Name=found['Name'], WithDecryption=True)
         secrets = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         result = { "Revisions":
             [{
                 'RevisionId': str(secret['Version']),
+                'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                 'Key': key,
                 'Value': secret['Value'],
-                'Scope': Contexts.translate_context(project, application, stage),
-                'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                'Scope': found['Scope'],
+                'Origin': found['Origin'],
                 'User': secret['LastModifiedUser'],
-                'Version': secret['Version'],
-                'Path': found[0]['Name'],
+                'Path': found['Name'],
             } for secret in secrets]
         }
 
@@ -188,15 +164,16 @@ class AwsSsmSecretProvider(SecretProvider):
                 raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
             # if len(found) > 1:
-            #     Logger.warn(f"More than one secret found at '{found[0]['Name']}'. The first one taken, and the rest were discarded.")
-            if not found[0]['Type'] == 'SecureString':
+            #     Logger.warn(f"More than one secret found at '{found['Name']}'. The first one taken, and the rest were discarded.")
+            if not found['Type'] == 'SecureString':
                 raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        Logger.info(f"Deleting SSM secret: path={found[0]['Name']}")
-        ssm.delete_parameter(Name=found[0]['Name'])
+        Logger.info(f"Deleting SSM secret: path={found['Name']}")
+        ssm.delete_parameter(Name=found['Name'])
         return {
                 'Key': key, 
-                'Scope': Contexts.translate_context(project, application, stage),
-                'Path': found[0]['Name'],
+                'Scope': found['Scope'],
+                'Origin': found['Origin'],
+                'Path': found['Name'],
                 }
 
 ###--------------------------------------------------------------------------------------------
