@@ -45,13 +45,13 @@ class TemplateManager():
         if '//' in key:
             raise DSOException(MESSAGES['InvalidKeyStr'].format(key, '//'))
 
-    def list(self, stage, uninherited=False, filter=None):
+    def list(self, stage, uninherited=False, include_contents=False, filter=None):
         project = Configs.project
         application = Configs.application
         provider = Providers.TemplateProvider()
         renderPaths = Configs.get_template_render_paths()
         Logger.info(f"Start listing templates: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        response = provider.list(project, application, stage, uninherited, filter)
+        response = provider.list(project, application, stage, uninherited, include_contents, filter)
         for template in response['Templates']:
             key = template['Key']
             if key in renderPaths:
@@ -105,13 +105,13 @@ class TemplateManager():
         Logger.info(f"Start getting the contents of template '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
         return provider.get(project, application, stage, key, revision)
 
-    def history(self, stage, key):
+    def history(self, stage, key, include_contents=False):
         # self.validate_key(key)
         project = Configs.project
         application = Configs.application
         provider = Providers.TemplateProvider()
         Logger.info(f"Start getting the history of template '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
-        return provider.history(project, application, stage, key)
+        return provider.history(project, application, stage, key, include_contents)
 
     def delete(self, stage, key):
         # self.validate_key(key)
@@ -180,7 +180,12 @@ class TemplateManager():
             with open(renderPath, 'w', encoding='utf-8') as f:
                 f.write(renderedContent)
             
-            rendered.append({'Key':key, 'RenderTo': renderPath})
+            rendered.append({
+                        'Key':key, 
+                        'Scope': item['Scope'],
+                        # 'Origin': item['Origin'],
+                        'RenderTo': renderPath,
+                        })
 
         return rendered
 
